package com.link.core.kafka.dynamickafka;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.function.Supplier;
import javax.annotation.Resource;
import org.apache.kafka.common.serialization.StringDeserializer;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ConfigurableApplicationContext;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.kafka.config.ConcurrentKafkaListenerContainerFactory;
import org.springframework.kafka.core.ConsumerFactory;
import org.springframework.kafka.core.DefaultKafkaConsumerFactory;
import org.springframework.kafka.listener.ContainerProperties.AckMode;

@Configuration
public class DynamicKafkaConsumerConfiguration {
   private static final Logger logger = LogManager.getLogger(DynamicKafkaConsumerConfiguration.class);
   public static final String DYNAMIC_KAFKA_CONSUMER_BEAN = "dynamicKafkaConsumerFactories";
   @Resource
   private DynamicKafkaProps dynamicKafkaProps;
   @Resource
   private ApplicationContext applicationContext;

   @Bean
   public void dynamicKafkaConsumerFactories() {
      if (this.dynamicKafkaProps != null && this.dynamicKafkaProps.getEnable()) {
         this.setAutoCommitFactory();
         this.setManualCommitFactory();
      }

   }

   private void setAutoCommitFactory() {
      ConfigurableApplicationContext context = (ConfigurableApplicationContext)this.applicationContext;
      DefaultListableBeanFactory beanFactory = (DefaultListableBeanFactory)context.getBeanFactory();
      Map<String, ConsumerFactory<String, String>> factories = this.dynamicConsumerFactory();
      Iterator var4 = factories.keySet().iterator();

      while(var4.hasNext()) {
         String key = (String)var4.next();
         ConsumerFactory<String, String> factory = (ConsumerFactory)factories.get(key);
         BeanDefinitionBuilder beanDefinitionBuilder = BeanDefinitionBuilder.rootBeanDefinition(ConcurrentKafkaListenerContainerFactory.class);
         beanDefinitionBuilder.addPropertyValue("consumerFactory", factory);
         beanDefinitionBuilder.addPropertyValue("batchListener", true);
         beanDefinitionBuilder.addPropertyValue("autoStartup", false);
         String beanName = key + "KafkaListenerFactory";
         beanFactory.registerBeanDefinition(beanName, beanDefinitionBuilder.getBeanDefinition());
      }

   }

   private void setManualCommitFactory() {
      try {
         ConfigurableApplicationContext context = (ConfigurableApplicationContext)this.applicationContext;
         DefaultListableBeanFactory beanFactory = (DefaultListableBeanFactory)context.getBeanFactory();
         Map<String, ConcurrentKafkaListenerContainerFactory<String, String>> concurrentKafkaListenerContainerFactoryMap = this.dynamicConcurrentKafkaListenerContainerFactory();
         Iterator var4 = concurrentKafkaListenerContainerFactoryMap.keySet().iterator();

         while(var4.hasNext()) {
            String key = (String)var4.next();
            String beanName = key + "Manual" + "KafkaListenerFactory";
            ConcurrentKafkaListenerContainerFactory<String, String> concurrentKafkaListenerContainerFactory = (ConcurrentKafkaListenerContainerFactory)concurrentKafkaListenerContainerFactoryMap.get(key);
            Supplier<ConcurrentKafkaListenerContainerFactory> factorySupplier = () -> {
               ConcurrentKafkaListenerContainerFactory<String, String> containerFactory = new ConcurrentKafkaListenerContainerFactory();
               containerFactory.setConsumerFactory(concurrentKafkaListenerContainerFactory.getConsumerFactory());
               containerFactory.getContainerProperties().setAckMode(AckMode.MANUAL_IMMEDIATE);
               containerFactory.setBatchListener(true);
               containerFactory.setAutoStartup(false);
               return containerFactory;
            };
            BeanDefinitionBuilder beanDefinitionBuilder = BeanDefinitionBuilder.genericBeanDefinition(ConcurrentKafkaListenerContainerFactory.class, factorySupplier);
            beanFactory.registerBeanDefinition(beanName, beanDefinitionBuilder.getBeanDefinition());
         }
      } catch (Exception var10) {
         Exception e = var10;
         logger.error("初始化手动提交Kafka失败：" + e);
      }

   }

   public Map<String, ConsumerFactory<String, String>> dynamicConsumerFactory() {
      Map<String, ConsumerFactory<String, String>> consumerFactoryMap = new HashMap(2);
      Map<String, Map<String, Object>> configs = this.dynamicConsumerConfigs();
      Iterator var3 = configs.keySet().iterator();

      while(var3.hasNext()) {
         String name = (String)var3.next();
         ConsumerFactory<String, String> factory = new DefaultKafkaConsumerFactory((Map)configs.get(name));
         consumerFactoryMap.put(name, factory);
      }

      return consumerFactoryMap;
   }

   public Map<String, ConcurrentKafkaListenerContainerFactory<String, String>> dynamicConcurrentKafkaListenerContainerFactory() {
      Map<String, ConcurrentKafkaListenerContainerFactory<String, String>> concurrentKafkaListenerContainerFactoryMap = new HashMap(2);
      Map<String, Map<String, Object>> configs = this.dynamicConsumerConfigs();
      Iterator var3 = configs.keySet().iterator();

      while(var3.hasNext()) {
         String name = (String)var3.next();
         ConsumerFactory<String, String> consumerFactory = new DefaultKafkaConsumerFactory((Map)configs.get(name));
         ConcurrentKafkaListenerContainerFactory<String, String> factory = new ConcurrentKafkaListenerContainerFactory();
         factory.setConsumerFactory(consumerFactory);
         concurrentKafkaListenerContainerFactoryMap.put(name, factory);
      }

      return concurrentKafkaListenerContainerFactoryMap;
   }

   public Map<String, Map<String, Object>> dynamicConsumerConfigs() {
      Map<String, Map<String, Object>> configMap = new HashMap(2);
      List<String> names = this.dynamicKafkaProps.getName();

      for(int i = 0; i < names.size(); ++i) {
         Map<String, Object> props = new HashMap();
         props.put("bootstrap.servers", this.dynamicKafkaProps.getConsumerServers().get(i));
         props.put("group.id", "LINK");
         props.put("enable.auto.commit", false);
         props.put("max.poll.records", "100");
         props.put("auto.commit.interval.ms", "100");
         props.put("session.timeout.ms", "15000");
         props.put("key.deserializer", StringDeserializer.class);
         props.put("value.deserializer", StringDeserializer.class);
         props.put("auto.offset.reset", "latest");
         String name = (String)this.dynamicKafkaProps.getName().get(i);
         configMap.put(name, props);
      }

      return configMap;
   }
}
